import common_utils
import json
import xmltodict
from xml.dom.minidom import parseString
from xml.dom.minidom import getDOMImplementation
import xml.etree.ElementTree as ET
from collections import OrderedDict
import os
import re
'''
{        
            "hbaddrs1":[
                {
                    "ip":"192.168.100.187",
                    "nodeid": "ns187"
                },
                {
                    "ip":"192.168.100.188",
                    "nodeid": "ns188"
                },   
                {
                    "ip":"192.168.100.189",
                    "nodeid": "ns189"
                }
            ],
       
            "hbaddrs2":[
                {
                    "ip":"192.168.100.187",
                    "nodeid": "ns187"
                },
                {
                    "ip":"192.168.100.188",
                    "nodeid": "ns188"
                },
                {
                    "ip":"192.168.100.188",
                    "nodeid": "ns189"
                }
            ],
            "hbaddrs2_enabled":  1
}
'''


def get_node_list():
    # 读取文件
    filename = "/etc/corosync/corosync.conf"
    try:
        read = open(filename)
    except:
        return {
            'action': False,
            'error': "File " + filename + " doesn't exist!"
        }
    node_list = []  # 存储文件内数据
    line = read.readline()  # 按行读取文件内容
    while line and not "nodelist {" in line:
        line = read.readline()
    # 读到nodelist
    # node_list.append(line)
    while line:
        if "quorum {" in line:
            break
        node_list.append(line.strip())
        line = read.readline()
    read.close
    # 修改格式
    while "nodelist {" in node_list:
        node_list.remove("nodelist {")
    while "" in node_list:
        node_list.remove("")
    while "}" in node_list:
        node_list.remove("}")
    return node_list

    # def get_hb_info():
    # cmd_str = "crm_node -l" #获取node节点信息
    # status, output = common_utils.run_cmd(cmd_str)
    # node_data = str(output).split("\n")
    # data = {}
    # hbaddrs1 = []
    # hbaddrs2 = []
    # nid = []
    # for i in node_data:
    #     ip1 = {}
    #     ip2 = {}
    #     node = i.split(" ")
    #     # print(node)
    #     nid.append(node[0])
    #     ip1['nodeid'] = node[1]
    #     ip2['nodeid'] = node[1]
    #     for i in nid:
    #         status, output = common_utils.run_cmd("corosync-cfgtool -a" + str(i))
    #     ip = str(output).split(" ")#ip
    #     if len(ip) <= 1:
    #         ip1['ip'] = ip[0]
    #     if len(ip) >= 1:
    #         ip1['ip'] = ip[0]
    #         ip2['ip'] = ip[1]
    #     hbaddrs1.append(ip1)
    #     hbaddrs2.append(ip2)
    # data["hbaddrs1"] = hbaddrs1
    # data["hbaddrs2"] = hbaddrs2
    # data["hbaddrs2_enabled"] = 1
    # if status != 0:
    #     return {'action': False, 'error': _(output)}
    # else:
    #     return {'action': True, "data": data}


# 判断集群是否存在
def is_cluster_exist():
    return os.path.exists("/etc/corosync/corosync.conf")


# 通过/var/lib/pcsd/known-hosts获取hb信息
def get_hb_by_hosts():
    cmd_str = "cat /var/lib/pcsd/known-hosts"
    status, output = common_utils.run_cmd(cmd_str)
    if status != 0:
        return {
            "action":
            False,
            "error":
            _("No node in the Cluster, please run \'pcs host auth $nodename\' to add node"
              )
        }
    # 解析json为字典
    info = json.loads(str(output))
    ret = {}
    if "known_hosts" in info.keys():
        nodes = info.get("known_hosts", "")
        hbs = []
        for i in nodes.keys():
            hbs.append(i)
        hbs_list = []
        for j in hbs:
            prop = {}
            prop["ip"] = ""
            prop["nodeid"] = str(j)
            hbs_list.append(prop)
        hb_str = "hbaddrs1"
    ret[hb_str] = hbs_list
    data = {}
    data['action'] = True
    data['data'] = ret
    return data


def get_hb_conf():
    # 判断集群是否存在
    if is_cluster_exist():
        return get_hb_dict()
    else:
        return get_hb_by_hosts()


def get_hb_dict():
    node_list = get_node_list()
    """
    ['node {', 'ring0_addr: ha3', 'name: ha3', 'nodeid: 1', 'node {', 'ring0_addr: ha4', 'name: ha4', 'nodeid: 2']
    """
    nodes = []  #按照node存储为键值对
    index = 0
    while index < len(node_list):
        if node_list[index] == "node {":
            # 开始按照node {进行分割遍历每个node
            index += 1
            node_info = {}
            while index < len(node_list) and node_list[index] != "node {":
                d = {}
                n = []
                n = node_list[index].split(":")
                d[n[0]] = n[1].strip()
                node_info.update(d)
                index += 1
            nodes.append(node_info)
    """
    nodes: #字典
    [
        {'ring0_addr': 'ha3', 'name': 'ha3', 'nodeid': '1'}, 
        {'ring0_addr': 'ha4', 'name': 'ha4', 'nodeid': '2'}
    ]
    """
    res = {}
    # 形成指定格式
    for i in nodes:
        """
        {'ring0_addr': 'ha3', 'ring1_addr': 'ha4', 'name': 'ha3', 'nodeid': '1'}
        """
        name = i.get("name", "")
        for k, v in i.items():
            if k != "name" and k != "nodeid":
                # 只选取ip
                """
                {'ring0_addr': 'ha3'}
                """
                info = {}
                info["nodeid"] = name
                info["ip"] = v
                # 判断属于哪个心跳
                hb = str(k)
                if hb in res.keys():
                    # 如果存在，向其中添加
                    res[hb].append(info)
                else:
                    # 如果不存在，新建
                    res[hb] = []
                    res[hb].append(info)
    # 将res里的所有ringx_addr重命名为hbaddrsx
    ret = {}
    count = 0
    for key, value in res.items():
        count += 1
        hb_str = "hbaddrs" + str(count)
        ret[hb_str] = value
    data = {}
    data['action'] = True
    data['data'] = ret
    return data


def edit_hb_info(data):
    if data == {} or data is None:
        return {'action': False, 'error': _('No input data')}
    else:
        hb_info = {}
        if 'hbaddrs1' in data:
            hbaddrs1 = data['hbaddrs1']
            hb = ""
            for nodeinfo in hbaddrs1:
                nodeid = str(nodeinfo['nodeid'])
                nodeip = str(nodeinfo['ip'])
                if nodeid not in hb_info:
                    hb_info[nodeid] = []
                hb_info[nodeid].append(nodeip)
        if 'hbaddrs2' in data:
            hbaddrs2 = data['hbaddrs2']
            hb = ""
            for nodeinfo in hbaddrs2:
                nodeid = str(nodeinfo['nodeid'])
                nodeip = str(nodeinfo['ip'])
                if nodeid not in hb_info:
                    hb_info[nodeid] = []
                hb_info[nodeid].append(nodeip)
    cmd1 = "pcs cluster setup hacluster"
    cmd2 = " --force --start"
    cmd = cmd1
    for key, value in hb_info.items():
        cmd = cmd + " " + str(key)
        for v in value:
            addr = "addr=" + str(v)
            cmd = cmd + " " + addr
    cmd = cmd + cmd2  
    # pcs cluster setup hacluster ha1 addr=10.1.167.86 ha2 addr=10.1.167.81
    cmd_resource = "crm_mon -1 --as-xml"
    status, output = common_utils.run_cmd(cmd_resource)
    if status == 0:
        json_str = json.dumps(xmltodict.parse(output))
        json_dict = json.loads(json_str)
        run_resources = json_dict['crm_mon']['resources']
    else:
        run_resources = None
    if run_resources is None:
        status, output = common_utils.run_cmd("pcs cluster cib ra-cfg")
        status, output = common_utils.run_cmd("pcs resource cleanup")
        status, output = common_utils.run_cmd("pcs cluster stop --all")
        status, output = common_utils.run_cmd("pcs cluster destroy --all")
        status, output = common_utils.run_cmd(cmd)
        status, output = common_utils.run_cmd("pcs cluster cib-push ra-cfg")
        return {"action": True, "info": _("Change cluster success")}
    else:
        return {"action": False, "error": _("There are running resources in the cluster, please close first")}


if __name__ == '__main__':
    print(get_hb_dict())
