import common_utils
import xmltodict
import json
import node
import re
import ast
from xml.dom.minidom import parseString
import xml.etree.ElementTree as ET


def get_rsc_metas(rsc_class, rsc_type, rsc_provider):
    # item = {'rsc_class':'ocf', 'rsc_type':'IPaddr', 'rsc_provider': 'common'}
    data = {}
    prop = []
    actions = []
    class_name = str(rsc_class)
    type_ = str(rsc_type)
    provider_name = str(rsc_provider)
    cmd = ""
    resource_type = "crm_resource  --show-metadata "
    # crm_resource --show-metadata ocf:heartbeat:IPaddr
    if provider_name == "None":
        cmd = resource_type + class_name + ":" + type_
    else:
        cmd = resource_type + class_name + ":" + provider_name + ":" + type_
    status, output = common_utils.run_cmd(cmd)
    etree = ET.fromstring(output)  # 封装
    data_name = etree.get("name", "")  # 找跟目录下的name
    data['name'] = data_name
    parameter = etree.findall("./parameters/parameter")  # parameter路径
    for i in parameter:
        parameters = {
            "name": i.get("name", ""),
            "required": i.get("required", ""),
            "unique": i.get("unique", "")
        }
        content = i.find("content")
        cnt = {
            "default": content.get("default", ""),
            "type": content.get("type", "")
        }
        parameters['content'] = cnt
        if i.find("shortdesc"):
            parameters['shortdesc'] = i.find("shortdesc").text.replace(
                "\n", " ")
        else:
            parameters['shortdesc'] = ""
        if i.find("longdesc"):
            parameters['longdesc'] = i.find("longdesc").text.replace("\n", " ")
        else:
            parameters['longdesc'] = ""
        prop.append(parameters)
        if class_name == 'stonith':
            pcmk_host_list = {
                'content': {
                    'default': "",
                    'type': "string"
                },
                'longdesc': "A list of machines controlled by this device.",
                'name': "pcmk_host_list",
                'required': "1",
                'shortdesc': "",
                'unique': "",
                'value': ""
            }
            prop.append(pcmk_host_list)
    for action in etree.findall("./actions/action"):
        act = {}
        for key, value in action.items():
            act[str(key)] = str(value)
        actions.append(act)
    if etree.find('version'):
        data['version'] = etree.find('version').text
    else:
        data['version'] = ""
    if etree.find('longdesc'):
        data['longdesc'] = etree.find('longdesc').text.replace("\n", " ")
    else:
        data['longdesc'] = ""
    if etree.find('shortdesc'):
        data['shortdesc'] = etree.find('shortdesc').text.replace("\n", " ")
    else:
        data['shortdesc'] = ""
    data['parameters'] = prop
    data['actions'] = actions

    if type_ == "fence_sbd":
        w = data["parameters"]
        for i in w:
            ret = i.get("name", "")
            if ret == 'plug':
                w.remove(i)
                break
    cmd_describe = "pcs resource describe "
    if provider_name == "None":
        cmd_describe = cmd_describe + class_name + ":" + type_
    else:
        cmd_describe = cmd_describe + class_name + ":" + provider_name + ":" + type_
    status, output = common_utils.run_cmd(cmd_describe)
    xml_describe = re.split(r'\s+\n', output)
    if len(xml_describe) > 4:
        xml = xml_describe[1] + xml_describe[2] + xml_describe[3]
    elif len(xml_describe) > 3:
        xml = xml_describe[1] + xml_describe[2]
    else:
        xml = xml_describe[1]
    data["xml_str"] = xml
    # Resource options
    options = []
    for option in xml_describe:
        if "Resource options" in option:
            options = option.split("\n")[1:]
            break
    meta = []
    for option in options:
        arr = option.split(":")
        arr = str(arr)
        a = re.sub("\(.*\)", "", arr)
        name = (a[0:a.rfind(',', 1) + 1])
        name = name.replace(" ", "")
        value = a.split(",")[-1]
        arr = name + value
        meta.append(arr)
    for i in meta:
        v = ast.literal_eval(i)
        for j in data['parameters']:
            if j['name'] == v[0]:
                j['longdesc'] = v[1]
    return {'action': True, "data": data}


class HAPublic:
    def level_init(self):
        self.level_score = []  # list(self.level_score)
        nodeinfo = node.get_node_info()
        node_num = len(nodeinfo['data'])
        max = node_num - 1
        for i in range(max):
            self.level_score.append(str(16000 - 1000 * i))

        return

    def turn_score_to_level(self, score):
        self.level_init()
        if score in ["20000"
                     ]:  # ["INFINITY", "+INFINITY", "infinity", "+infinity"]:
            return "Master Node"

        if score in ["-INFINITY", "-infinity"]:
            return ("No Run Node")

        if score not in self.level_score:
            return score
        level = 1
        for s in self.level_score:
            if s == score:
                return ("Slave %d") % (level)
            level = level + 1

        return score

    def turn_level_to_score(self, level):
        self.level_init()
        if level == "Master Node":
            return "20000"  # "INFINITY"

        if level == ("No Run Node"):
            return "-INFINITY"

        if ("Slave ") not in level:
            return level
        else:
            index = int(level[len(str("Slave ")):])
            return self.level_score[index - 1]

    def add_score(self, score1, score2):
        if score1 in ["-INFINITY", "-infinity"
                      ] or score2 in ["-INFINITY", "-infinity"]:
            return str("-INFINITY")
        if score1 in [
                "INFINITY", "+INFINITY", "infinity", "+infinity"
        ] or score2 in ["INFINITY", "+INFINITY", "infinity", "+infinity"]:
            return str("INFINITY")
        return str(int(score1) + int(score2))


web_manager = HAPublic()


def get_sub_rscs(rsc_id):
    # 把运行状态信息进行变更
    #rsc_status = update_rsc_failed_info()
    rsc_status = get_all_rsc_status()
    # 获取错误运行资源信息
    failure_info = get_rsc_failed_message()

    status, stdout = common_utils.run_cmd("cibadmin --query --scope resources")
    json_str = json.dumps(xmltodict.parse(stdout))
    res_json = json.loads(json_str)['resources']
    rsc_type = get_rsc_type(rsc_id)
    rsc_info = {}
    rsc_info["id"] = rsc_id
    sub_rscs = []
    # 如果获取的是clone类型
    if rsc_type == "clone":
        node_info = node.get_node_info()
        node_num = len(node_info['data'])
        clone = res_json["clone"]
        clone_aim = {}
        # 查找该clone资源在什么位置，首先判断下是不是多个clone（是否是list），这样可以方便的去查找。
        if not isinstance(clone, list):
            clone_id = clone["@id"]
            if clone_id == rsc_id:
                clone_aim = clone
        else:
            for sub_clone in clone:
                clone_id = sub_clone["@id"]
                if clone_id == rsc_id:
                    clone_aim = sub_clone
        # 查找到目标资源的clone_aim,clone资源的对象可能是group和primitive两种类型，分析两种类型，并组装成需要的字符串
        if "group" in clone_aim:
            sub_rsc_id = clone_aim["group"]["@id"]
            sub_sub_rsc = []
            # clone资源的子资源可能是group资源，group资源也可能是单个资源或者多个资源
            if "primitive" in clone_aim["group"]:
                rsc_primitives = clone_aim["group"]["primitive"]
                # 显示时，子资源有针对节点的编号，从0开始，数目是节点数
                for i in range(0, node_num):
                    sub_rsc = {}
                    sub_rsc["status"] = "Not Running"
                    sub_rsc["running_node"] = []
                    sub_rsc["status_message"] = ""
                    sub_the_id = sub_rsc_id + ":" + str(i)
                    sub_rsc["id"] = sub_the_id
                    if sub_the_id in rsc_status:
                        sub_rsc["status"] = rsc_status[sub_the_id]["status"]
                        sub_rsc["running_node"] = rsc_status[sub_the_id][
                            "running_node"]
                    sub_rsc["type"] = "group"
                    sub_sub_rsc = []
                    if not isinstance(rsc_primitives, list):
                        primitive_info = {}
                        # 设置状态信息
                        primitive_info["status_message"] = ""
                        primitive_info["status"] = "Not Running"
                        primitive_info["running_node"] = []

                        primitive_info["id"] = rsc_primitives["@id"]
                        pri_id = primitive_info["id"] + ":" + str(i)
                        if pri_id in rsc_status:
                            primitive_info["status"] = rsc_status[pri_id][
                                "status"]
                            primitive_info["running_node"] = rsc_status[
                                pri_id]["running_node"]

                        primitive_info["svc"] = rsc_primitives["@type"]
                        primitive_info["type"] = "primitive"
                        sub_sub_rsc.append(primitive_info)
                        sub_rsc["subrscs"] = sub_sub_rsc
                    else:
                        for primitive in rsc_primitives:
                            pid = primitive["@id"] + ":" + str(i)
                            primitive_info = {}
                            primitive_info["id"] = pid
                            primitive_info["status"] = "Not Running"
                            primitive_info["status_message"] = ""
                            primitive_info["running_node"] = []

                            if pid in rsc_status:
                                primitive_info["status"] = rsc_status[pid][
                                    "status"]
                                primitive_info["running_node"] = rsc_status[
                                    pid]["running_node"]

                            primitive_info["svc"] = primitive["@type"]
                            primitive_info["type"] = "primitive"
                            sub_sub_rsc.append(primitive_info)
                        sub_rsc["subrscs"] = sub_sub_rsc
                    sub_rscs.append(sub_rsc)
        # 如果clone资源的子资源是primitive进行的处理
        if "primitive" in clone_aim:
            sub_rsc_id = clone_aim["primitive"]["@id"]
            for i in range(0, node_num):
                sub_rsc = {}

                sub_rsc["status"] = "Not Running"
                sub_rsc["running_node"] = []
                sub_rsc["status_message"] = ""

                sub_the_id = sub_rsc_id + ":" + str(i)
                sub_rsc["id"] = sub_the_id
                if sub_rsc["id"] in rsc_status:
                    sub_rsc["status"] = rsc_status[sub_the_id]["status"]
                    sub_rsc["running_node"] = rsc_status[sub_the_id][
                        "running_node"]

                # 判断失败信息
                if sub_rsc_id in failure_info:
                    sub_rsc["status_message"] = failure_info[sub_rsc_id][
                        'exitreason'] + " on " + str(
                            failure_info[sub_rsc_id]['node'])
                    sub_rsc["status"] = "Failed"
                    if sub_rsc_id in rsc_status:
                        sub_rsc["running_node"] = rsc_status[sub_rsc_id]['running_node'] 
                    else:
                        sub_rsc["running_node"] = []

                sub_rsc["type"] = "primitive"
                sub_rsc["svc"] = clone_aim["primitive"]["@type"]
                sub_rscs.append(sub_rsc)

    if rsc_type == "group":
        groups = res_json["group"]
        # 如果有多个组资源
        if isinstance(groups, list):
            for group in groups:
                if rsc_id == group['@id']:
                    rsc_primitives = group["primitive"]
                    sub_rsc = []
                    if not isinstance(rsc_primitives, list):
                        primitive_info = {}
                        primitive_info["status_message"] = ""
                        rsc_the_id = rsc_primitives["@id"]
                        primitive_info["id"] = rsc_the_id
                        if rsc_the_id in rsc_status:
                            primitive_info["status"] = rsc_status[rsc_the_id][
                                "status"]
                            primitive_info["running_node"] = rsc_status[
                                rsc_the_id]["running_node"]
                        else:
                            primitive_info["status"] = ""
                            primitive_info["running_node"] = []

                        # 判断失败信息
                        if rsc_the_id in failure_info:
                            primitive_info["status_message"] = failure_info[
                                rsc_the_id]['exitreason'] + " on " + str(
                                    failure_info[rsc_the_id]['node'])
                            primitive_info["status"] = "Failed"
                            if rsc_the_id in rsc_status:
                                primitive_info["running_node"] = rsc_status[rsc_the_id]['running_node']
                            else:
                                primitive_info["running_node"] = []

                        primitive_info["svc"] = rsc_primitives["@type"]
                        primitive_info["type"] = "primitive"
                        sub_rscs.append(primitive_info)
                    else:
                        for primitive in rsc_primitives:
                            primitive_info = {}
                            primitive_info["status_message"] = ""
                            primitive_the_id = primitive["@id"]
                            primitive_info["id"] = primitive_the_id
                            if primitive_the_id in rsc_status:
                                primitive_info["status"] = rsc_status[
                                    primitive_the_id]["status"]
                                primitive_info["running_node"] = rsc_status[
                                    primitive_the_id]["running_node"]
                            else:
                                primitive_info["status"] = ""
                                primitive_info["running_node"] = []

                                # 判断失败信息
                            if primitive_the_id in failure_info:
                                primitive_info[
                                    "status_message"] = failure_info[
                                        primitive_the_id][
                                            'exitreason'] + " on " + str(
                                                failure_info[primitive_the_id]
                                                ['node'])
                                primitive_info["status"] = "Failed"
                                if primitive_the_id in rsc_status:
                                    primitive_info["running_node"] = rsc_status[primitive_the_id]['running_node']
                                else:
                                    primitive_info["running_node"] = []

                            primitive_info["svc"] = primitive["@type"]
                            primitive_info["type"] = "primitive"
                            sub_rscs.append(primitive_info)
        # 如果只有一个组资源
        else:
            if rsc_id == groups['@id']:
                rsc_primitives = groups["primitive"]
                sub_rsc = []
                if not isinstance(rsc_primitives, list):
                    primitive_info = {}
                    primitive_info["status_message"] = ""
                    rsc_the_id = rsc_primitives["@id"]
                    primitive_info["id"] = rsc_the_id
                    if rsc_primitives["@id"] in rsc_status:
                        primitive_info["status"] = rsc_status[rsc_the_id][
                            "status"]
                        primitive_info["running_node"] = rsc_status[
                            rsc_the_id]["running_node"]
                    else:
                        primitive_info["status"] = ""
                        primitive_info["running_node"] = []

                    if rsc_the_id in failure_info:
                        primitive_info["status_message"] = failure_info[
                            rsc_the_id]['exitreason'] + " on " + str(
                                failure_info[rsc_the_id]['node'])
                        primitive_info["status"] = "Failed"
                        if rsc_the_id in rsc_status:
                            primitive_info["running_node"] = rsc_status[rsc_the_id]['running_node']
                        else:
                            primitive_info["running_node"] = []

                    primitive_info["svc"] = rsc_primitives["@type"]
                    primitive_info["type"] = "primitive"
                    sub_rscs.append(primitive_info)
                else:
                    for primitive in rsc_primitives:
                        primitive_info = {}
                        primitive_info["status_message"] = ""
                        primitive_the_id = primitive["@id"]
                        primitive_info["id"] = primitive_the_id

                        if primitive_the_id in rsc_status:
                            primitive_info["status"] = rsc_status[
                                primitive_the_id]["status"]
                            primitive_info["running_node"] = rsc_status[
                                primitive_the_id]["running_node"]
                        else:
                            primitive_info["status"] = ""
                            primitive_info["running_node"] = []

                        if primitive_the_id in failure_info:
                            primitive_info["status_message"] = failure_info[
                                primitive_the_id]['exitreason'] + " on " + str(
                                    failure_info[primitive_the_id]['node'])
                            primitive_info["status"] = "Failed"
                            if primitive_the_id in rsc_status:
                                primitive_info["running_node"] = rsc_status[primitive_the_id]['running_node']
                            else:
                                primitive_info["running_node"] = []

                        primitive_info["svc"] = primitive["@type"]
                        primitive_info["type"] = "primitive"
                        sub_rscs.append(primitive_info)

    rsc_info["subrscs"] = sub_rscs
    return rsc_info


# 获取顶级显示的资源
def get_top_rsc():
    status, stdout = common_utils.run_cmd("cibadmin --query --scope resources")
    json_str = json.dumps(xmltodict.parse(stdout))
    res_json = json.loads(json_str)['resources']
    top_rsc = []
    if res_json is None:
        return top_rsc
    if "clone" in res_json:
        clone = res_json["clone"]
        if not isinstance(clone, list):
            top_rsc.append(clone['@id'])
        else:
            for rsc in clone:
                top_rsc.append(rsc['@id'])

    if "group" in res_json:
        group = res_json["group"]
        if not isinstance(group, list):
            top_rsc.append(group['@id'])
        else:
            for rsc in group:
                top_rsc.append(rsc['@id'])

    if "primitive" in res_json:
        primitive = res_json["primitive"]
        if not isinstance(primitive, list):
            top_rsc.append(primitive['@id'])
        else:
            for rsc in primitive:
                top_rsc.append(rsc['@id'])
    return top_rsc


def get_all_constraints():
    #rsc_status = update_rsc_failed_info()
    rsc_status = get_all_rsc_status()
    data = {}
    status, stdout = common_utils.run_cmd("cibadmin -Q")
    xml_nodes = parseString(stdout).documentElement
    # 针对顶层资源进行遍历
    for rsc_id in get_top_rsc():
        rsc = rsc_id.split(":")[0]
        data[rsc] = {}
        if rsc not in rsc_status:
            data[rsc]["status"] = "Stopped"
            data[rsc]["status_message"] = ""
            data[rsc]['running_node'] = []
        else:
            rsc_info = rsc_status[rsc]
            data[rsc]["status"] = "Running"
            data[rsc]["status_message"] = ""
            data[rsc]["running_node"] = rsc_info["running_node"]

        data[rsc]["before_rscs"] = []
        data[rsc]["after_rscs"] = []
        data[rsc]["same_node"] = []
        data[rsc]["diff_node"] = []
        data[rsc]['location'] = []

    constraints = xml_nodes.getElementsByTagName('constraints')[0]
    # location
    for location in constraints.getElementsByTagName('rsc_location'):
        if location.getAttribute('id').startswith('cli-prefer-'):
            continue
        node = location.getAttribute('node')
        rsc_id = location.getAttribute('rsc')
        score = location.getAttribute('score')
        location_single = {}
        location_single['node'] = node
        location_single['level'] = web_manager.turn_score_to_level(score)
        # location_list.append(location_single)
        if rsc_id in data.keys():
            data[rsc_id]["location"].append(location_single)

    # order
    for node in constraints.getElementsByTagName('rsc_order'):
        first_rsc_id = node.getAttribute("first")
        then_rsc_id = node.getAttribute('then')
        try:
            score = node.getAttribute('score')
            if score is None or len(score) == 0:
                score = 'infinity'
            if score not in ["INFINITY", "+INFINITY", "infinity", "+infinity"]:
                continue
        except:
            pass
        if first_rsc_id in data.keys():
            data[first_rsc_id]["after_rscs"].append({'id': then_rsc_id})

        if then_rsc_id in data.keys():
            data[then_rsc_id]["before_rscs"].append({'id': first_rsc_id})

    # colocation
    for node in constraints.getElementsByTagName('rsc_colocation'):
        first_rsc_id = node.getAttribute("rsc")
        with_rsc_id = node.getAttribute('with-rsc')
        try:
            score = node.getAttribute('score')
            if score in ["INFINITY", "+INFINITY", "infinity", "+infinity"]:
                rsc = {}
                rsc['rsc'] = first_rsc_id
                rsc['with_rsc'] = with_rsc_id
                data[first_rsc_id]["same_node"].append(rsc)
                data[with_rsc_id]["same_node"].append(rsc)
            elif score in ["-INFINITY", "-infinity"]:
                rsc = {}
                rsc['rsc'] = first_rsc_id
                rsc['with_rsc'] = with_rsc_id
                data[first_rsc_id]["diff_node"].append(rsc)
                data[with_rsc_id]["diff_node"].append(rsc)
        except:
            pass

    failure_info = get_rsc_failed_message()

    constraints = []
    for rsc_id in data.keys():
        constraint = {}
        constraint['id'] = rsc_id
        rsc_id_split = rsc_id.split(':')[0]
        if rsc_id not in rsc_status:
            if rsc_id_split not in failure_info:
                constraint["status"] = "Stopped"
                constraint["status_message"] = ""
                constraint['running_node'] = []
            elif not rsc_id.endswith('-clone'):
                constraint["status"] = "Failed"
                constraint["status_message"] = failure_info[rsc_id_split][
                    'exitreason'] + " on " + str(
                        failure_info[rsc_id_split]['node'])
                constraint['running_node'] = []
        else:
            rsc_info = rsc_status[rsc_id]
            constraint["status"] = rsc_info["status"]
            constraint["status_message"] = ""
            constraint["running_node"] = rsc_info["running_node"]

        colocation = {}
        colocation['same_node'] = data[rsc_id]["same_node"]
        colocation['diff_node'] = data[rsc_id]["diff_node"]
        colocation['same_node_num'] = len(colocation['same_node'])
        colocation['diff_node_num'] = len(colocation['diff_node'])
        order = {}
        order['before_rscs'] = data[rsc_id]["before_rscs"]
        order['after_rscs'] = data[rsc_id]["after_rscs"]
        order['before_rscs_num'] = len(order['before_rscs'])
        order['after_rscs_num'] = len(order['after_rscs'])
        constraint['location'] = data[rsc_id]["location"]
        constraint['colocation'] = colocation
        constraint['order'] = order
        constraints.append(constraint)
    return {'action': True, 'data': constraints}


# 获取可迁移资源
def get_all_migrate_rscs():
    status, stdout = common_utils.run_cmd("cibadmin -Q")
    xml_nodes = parseString(stdout).documentElement
    rsc_locations = {}
    for rsc_location in xml_nodes.getElementsByTagName("rsc_location"):
        id = str(rsc_location.getAttribute("id"))
        rsc_locations[id] = rsc_location
    migrate_ids = {}
    for id in rsc_locations:
        for prefix in ["cli-prefer-", "cli-standby-"]:
            if id.startswith(prefix):
                split_id = id.split(prefix)
                if len(split_id) > 1:
                    rsc_id = split_id[1]
                    if not rsc_id in migrate_ids:
                        migrate_ids[rsc_id] = []
                    migrate_ids[rsc_id].append(id)
    rsc_list = []
    if migrate_ids == {}:
        rsc_list = []
    else:
        rsc_list = list(migrate_ids.keys())
    return rsc_list


# 获取资源的类型
def get_rsc_type(rsc_id):
    cmd = "cibadmin --query --scope resources|grep 'id=\"" + str(
        rsc_id) + "\"'"
    status, output = common_utils.run_cmd(cmd)
    type_str = output.strip()
    if status == 0:
        rsc_type = type_str.split(' ')[0].replace('<', "")
        return rsc_type

    return None


def get_rsc_status(rsc_info):
    '''
   {
    "@id":"test3",
    "@resource_agent":"ocf::pacemaker:Dummy",
    "@role":"Started",
    "@active":"true",
    "@orphaned":"false",
    "@blocked":"false",
    "@managed":"true",
    "@failed":"false",
    "@failure_ignored":"false",
    "@nodes_running_on":"1",
    "node":{
            "@name":"ha3",
            "@id":"1",
            "@cached":"false"
           }
    }
   '''

    rsc_id = rsc_info['@id']
    failed_info = get_rsc_failed_message()
    if rsc_id in failed_info:
        return "Failed"

    if rsc_info['@managed'] == "false":
        return "Unmanaged"

    if rsc_info['@failed'] == "true":
        return "Failed"

    if '@role' in rsc_info:
        if rsc_info['@role'] == "Started":
            return "Running"
        if rsc_info['@role'] == "Stopped":
            return "Not Running"

    return "Running"


def get_rsc_failed_message():

    cmd = "crm_mon -1 --as-xml"

    status, stdout = common_utils.run_cmd(cmd)
    json_str = json.dumps(xmltodict.parse(stdout))
    json_dict = json.loads(json_str)
    failure_info = {}
    if 'failures' in json_dict['crm_mon']:
        failures = json_dict['crm_mon']['failures']['failure']
        if isinstance(failures, list):
            for failure in failures:
                info_failure = {}
                rsc_id = str(failure['@op_key']).split("_stop_")[0].split("_start_")[0].split("_monitor_")[0]
                node = failure['@node']
                exitreason = failure['@exitreason']
                info_failure['node'] = node
                info_failure['exitreason'] = exitreason
                failure_info[rsc_id] = info_failure
        else:
            info_failure = {}
            rsc_id = str(failures['@op_key']).split("_stop_")[0].split("_start_")[0].split("_monitor_")[0]
            node = failures['@node']
            exitreason = failures['@exitreason']
            info_failure['node'] = node
            info_failure['exitreason'] = exitreason
            failure_info[rsc_id] = info_failure
    return failure_info


def update_rsc_failed_info():

    rsc_info = get_all_rsc_status()

    cmd = "crm_mon -1 --as-xml"
    status, stdout = common_utils.run_cmd(cmd)
    json_str = json.dumps(xmltodict.parse(stdout))
    json_dict = json.loads(json_str)

    failures = json_dict['crm_mon']['failures']['failure']
    failure_info = {}
    for failure in failures:
        info_failure = {}
        rsc_id = str(failure['@op_key']).split("_stop_")[0].split("_start_")[0].split("_monitor_")[0]
        node = failure['@node']
        exitreason = failure['@exitreason']
        info_failure['node'] = node
        info_failure['exitreason'] = exitreason
        failure_info[rsc_id] = info_failure

    for rsc in rsc_info:
        rsc_id = rsc.split(':')[0]
        if rsc_id in failure_info:
            rsc_info[rsc]['status_message'] = failure_info[rsc_id][
                'exitreason'] + " on " + str(failure_info[rsc_id]['node'])
            rsc_info[rsc]['status'] = "Failed"

    return rsc_info


# 获取资源的运行状态
def get_all_rsc_status():
    '''
        infos = {
          0:_('Running'),
          1:_('Not Running'),
          2:_('Unmanaged'),
          3:_('Failed'),
          4:_('Stop Failed'),
          5:_('running (Master)'),
          6:_('running (Slave)')}
          rsc_info = {
             "hj1": {"status": 0 , "status_message": "test", running_node: []}
             "hj2": {"status": 0 , "status_message": "test", running_node: []}
          }
    '''

    rsc_info = {}
    cmd = "crm_mon -1 --as-xml"
    status, stdout = common_utils.run_cmd(cmd)
    json_str = json.dumps(xmltodict.parse(stdout))
    json_dict = json.loads(json_str)
    all_run_rscs = []
    resources_clone = {}
    resources_group = {}
    resources_primitive = {}

    # 获取运行的三类资源
    run_resources = json_dict['crm_mon']['resources']
    if run_resources is None:
        return {}
    if "clone" in run_resources:
        resources_clone = run_resources["clone"]
    if "group" in run_resources:
        resources_group = run_resources["group"]
    if "resource" in run_resources:
        resources_primitive = run_resources["resource"]

    if resources_clone != {}:
        # 如果运行着多个clone
        if isinstance(resources_clone, list):
            for resource in resources_clone:
                all_run_rscs.append(resource["@id"])
                # 子资源是普通资源
                if "resource" in resource:
                    sub_resources = resource["resource"]
                    index = 0
                    clone_run_nodes = []
                    clone_info = {}
                    for sub_resource in sub_resources:
                        all_run_rscs.append(sub_resource["@id"])
                        info = {}
                        info["status"] = get_rsc_status(sub_resource)
                        info["status_message"] = ""
                        info["running_node"] = []
                        if "node" in sub_resource:
                            nodename = sub_resource["node"]["@name"]
                        id = str(sub_resource["@id"]) + ":" + str(index)
                        index = index + 1
                        info["running_node"].append(nodename)
                        rsc_info[id] = info
                        clone_run_nodes.append(nodename)
                    clone_info["status"] = get_rsc_status(resource)
                    clone_info["status_message"] = ""
                    clone_info["running_node"] = clone_run_nodes
                    clone_id = resource["@id"]
                    rsc_info[clone_id] = clone_info
                # 子资源是group
                if 'group' in resource:
                    clone_run_nodes = []
                    clone_info = {}
                    sub_group_resources = resource['group']
                    for sub_resource in sub_group_resources:
                        sub_resource_id = sub_resource["@id"]
                        all_run_rscs.append(sub_resource_id)
                        group_info = {}
                        group_run_nodes = []
                        if 'resource' in sub_resource:
                            inner_resources = sub_resource['resource']
                            group_info["status"] = "Not Running"
                            if not isinstance(inner_resources, list):
                                inner_resources_id = inner_resources['@id']
                                info = {}
                                info["status"] = get_rsc_status(
                                    inner_resources)
                                info["status_message"] = ""
                                info["running_node"] = []
                                if "node" in inner_resources:
                                    nodename = inner_resources["node"]["@name"]
                                    clone_run_nodes.append(nodename)
                                    info["running_node"].append(nodename)
                                    group_run_nodes.append(nodename)
                                    group_info["status"] = "Running"
                                father_id = sub_resource_id.split(':')[1]
                                id = str(inner_resources_id) + ":" + str(
                                    father_id)
                                rsc_info[id] = info
                            else:
                                for inner_resource in inner_resources:
                                    inner_resource_id = inner_resource['@id']
                                    info = {}
                                    # 先把子组的状态设置为Stopped
                                    group_info["status"] = "Stopped"
                                    info["status"] = get_rsc_status(
                                        inner_resource)
                                    # 组内只要有一个子资源运行正常，可以理解为组是运行的
                                    if info["status"] == "Running":
                                        group_info["status"] = "Running"
                                        # 添加到运行资源组
                                        if sub_resource_id not in all_run_rscs:
                                            all_run_rscs.append(
                                                sub_resource_id)
                                    info["status_message"] = ""
                                    info["running_node"] = []
                                    if "node" in inner_resource:
                                        nodename = inner_resource["node"][
                                            "@name"]
                                        clone_run_nodes.append(nodename)
                                        info["running_node"].append(nodename)
                                    father_id = sub_resource_id.split(':')[1]
                                    id = str(inner_resource_id) + ":" + str(
                                        father_id)
                                    # 添加运行资源到all_run_rscs
                                    if id not in all_run_rscs:
                                        all_run_rscs.append(id)
                                    rsc_info[id] = info
                                    if nodename not in group_run_nodes:
                                        group_run_nodes.append(nodename)
                        group_info["status_message"] = ""
                        group_info["running_node"] = list(set(group_run_nodes))
                        group_id = sub_resource_id
                        rsc_info[group_id] = group_info
                    clone_info["status"] = get_rsc_status(resource)
                    clone_info["status_message"] = ""
                    clone_info["running_node"] = list(set(clone_run_nodes))
                    clone_id = resource["@id"]
                    rsc_info[clone_id] = clone_info
        # 如果运行着单个clone
        else:
            # 如果clone的是组资源
            if 'group' in resources_clone:
                resource_clone_groups = resources_clone['group']
                clone_info = {}
                clone_running_nodes = []
                clone_info["status"] = "Not Running"

                for resource_clone_group in resource_clone_groups:
                    all_run_rscs.append(resource_clone_group['@id'])
                    if 'resource' in resource_clone_group:
                        sub_resources = resource_clone_group['resource']
                        index = 0
                        clone_group_run_nodes = []
                        clone_group_info = {}
                        clone_group_info["status"] = "Not Running"

                        sub_group_id = resource_clone_group['@id']
                        index = sub_group_id.split(":")[1]
                        res_num = int(
                            resource_clone_group['@number_resources'])
                        # 如果组内有多个资源
                        if res_num >= 2:
                            for sub_resource in sub_resources:
                                info = {}
                                info["status"] = get_rsc_status(sub_resource)
                                info["status_message"] = ""
                                info["running_node"] = []
                                id = str(
                                    sub_resource["@id"]) + ":" + str(index)

                                if "node" in sub_resource:
                                    nodename = sub_resource["node"]["@name"]
                                    info["running_node"].append(nodename)
                                    all_run_rscs.append(id)
                                    if nodename not in clone_group_run_nodes:
                                        clone_group_run_nodes.append(nodename)
                                    clone_group_info["status"] = "Running"
                                    clone_info["status"] = "Running"
                                    clone_running_nodes.append(nodename)

                                rsc_info[id] = info
                        # 如果组内只有单个资源
                        else:
                            info = {}
                            info["status"] = get_rsc_status(sub_resources)
                            info["status_message"] = ""
                            info["running_node"] = []
                            id = str(sub_resources["@id"]) + ":" + str(index)
                            if "node" in sub_resources:
                                nodename = sub_resources["node"]["@name"]
                                info["running_node"].append(nodename)
                                all_run_rscs.append(id)
                                if nodename not in clone_group_run_nodes:
                                    clone_group_run_nodes.append(nodename)
                                clone_group_info["status"] = "Running"
                                clone_info["status"] = "Running"
                                clone_running_nodes.append(nodename)

                            rsc_info[id] = info

                    clone_group_info["status_message"] = ""
                    clone_group_info["running_node"] = clone_group_run_nodes
                    rsc_info[sub_group_id] = clone_group_info

                # 存储克隆组资源运行信息
                clone_info["status_message"] = ""
                # 去除重复资源
                clone_info["running_node"] = list(set(clone_running_nodes))
                clone_id = resources_clone["@id"]
                rsc_info[clone_id] = clone_info
            # 如果clone的是普通资源
            if 'resource' in resources_clone:
                sub_resources = resources_clone["resource"]
                index = 0
                clone_run_nodes = []
                clone_info = {}
                for sub_resource in sub_resources:
                    all_run_rscs.append(sub_resource["@id"])
                    info = {}
                    info["status"] = get_rsc_status(sub_resource)
                    info["status_message"] = ""
                    info["running_node"] = []
                    if "node" in sub_resource:
                        nodename = sub_resource["node"]["@name"]
                    id = str(sub_resource["@id"]) + ":" + str(index)
                    index = index + 1
                    info["running_node"].append(nodename)
                    rsc_info[id] = info
                    clone_run_nodes.append(nodename)
                clone_info["status"] = get_rsc_status(resources_clone)
                clone_info["status_message"] = ""
                clone_info["running_node"] = clone_run_nodes
                clone_id = resources_clone["@id"]
                rsc_info[clone_id] = clone_info
    if resources_group != {}:
        # 如果运行着多个group
        if isinstance(resources_group, list):
            for resource in resources_group:
                all_run_rscs.append(resource["@id"])
                sub_resources = resource['resource']
                # 如果组资源包括多个子资源
                if isinstance(sub_resources, list):
                    group_run_nodes = []
                    group_info = {}
                    group_info["status"] = "Not Running"
                    for sub_resource in sub_resources:
                        all_run_rscs.append(sub_resource["@id"])
                        info = {}
                        info["status"] = get_rsc_status(sub_resource)
                        info["status_message"] = ""
                        info["running_node"] = []
                        if "node" in sub_resource:
                            nodename = sub_resource["node"]["@name"]
                            if nodename not in info["running_node"]:
                                info["running_node"].append(nodename)
                            if nodename not in group_run_nodes:
                                group_run_nodes.append(nodename)
                            group_info["status"] = "Running"
                        id = str(sub_resource["@id"])
                        rsc_info[id] = info
                    group_info["status_message"] = ""
                    group_info["running_node"] = group_run_nodes
                    group_id = resource["@id"]
                    rsc_info[group_id] = group_info
                else:
                    info = {}
                    group_info = {}
                    group_run_nodes = []
                    group_info["status"] = "Not Running"
                    info["status"] = get_rsc_status(sub_resources)
                    info["status_message"] = ""
                    info["running_node"] = []
                    if "node" in sub_resources:
                        nodename = sub_resources["node"]["@name"]
                        all_run_rscs.append(sub_resources["@id"])
                        if nodename not in group_run_nodes:
                            group_run_nodes.append(nodename)
                            group_info["status"] = "Running"
                        if nodename not in info["running_node"]:
                            info["running_node"].append(nodename)
                    id = str(sub_resources["@id"])
                    rsc_info[id] = info
                    group_id = resource["@id"]
                    group_info["status_message"] = ""
                    group_info["running_node"] = group_run_nodes
                    rsc_info[group_id] = group_info
        # 如果运行着单个组资源
        else:
            group_id = resources_group["@id"]
            all_run_rscs.append(resources_group["@id"])
            sub_resources = resources_group['resource']
            group_run_nodes = []
            group_info = {}
            flag = 0
            # 如果组资源包括多个子资源
            if isinstance(sub_resources, list):
                group_info["status"] = "Not Running"
                group_info["running_node"] = []
                for sub_resource in sub_resources:
                    all_run_rscs.append(sub_resource["@id"])
                    info = {}
                    info["status"] = get_rsc_status(sub_resource)
                    info["status_message"] = ""
                    info["running_node"] = []
                    id = str(sub_resource["@id"])
                    if "node" in sub_resource:
                        if isinstance(sub_resource["node"], list):
                            for i in sub_resource["node"]:
                                nodename = i["@name"]
                                if nodename not in info["running_node"]:
                                    info["running_node"].append(nodename)
                                if nodename not in group_info["running_node"]:
                                    group_info["running_node"].append(nodename)
                                    group_info["status"] = "Running"
                        else:
                            nodename = sub_resource["node"]["@name"]
                            if nodename not in info["running_node"]:
                                info["running_node"].append(nodename)
                            if nodename not in group_info["running_node"]:
                                group_info["running_node"].append(nodename)
                                group_info["status"] = "Running"
                    rsc_info[id] = info
                group_info["status_message"] = ""
                group_id = resources_group["@id"]
                rsc_info[group_id] = group_info
            else:
                all_run_rscs.append(sub_resources["@id"])
                info = {}
                info["status"] = get_rsc_status(sub_resources)
                info["status_message"] = ""
                info["running_node"] = []
                if "node" in sub_resources:
                    nodename = sub_resources["node"]["@name"]
                else:
                    flag = 1
                id = str(sub_resources["@id"])
                info["running_node"].append(nodename)
                rsc_info[id] = info
                group_info["status_message"] = ""
                group_info["running_node"] = []
                if flag == 1:
                    group_info["status"] = "Not Running"
                else:
                    group_info["status"] = "Running"
                    group_info["running_node"].append(nodename)
                group_id = resources_group["@id"]
                rsc_info[group_id] = group_info
    if resources_primitive != {}:
        # 如果运行着多个普通资源
        if isinstance(resources_primitive, list):
            for resource in resources_primitive:
                running_node = []
                resource_info = {}
                resource_info["status"] = get_rsc_status(resource)
                if "node" in resource:
                    # 有时候资源运行故障会出现 FAILED (blocked)[ha3 ha4]
                    nodes = resource["node"]
                    if not isinstance(nodes, list):
                        nodename = nodes['@name']
                        running_node.append(nodename)
                    else:
                        for node in nodes:
                            nodename = node['@name']
                            running_node.append(nodename)

                resource_info["running_node"] = running_node
                resource_info["status_message"] = ""
                resource_id = resource["@id"]
                all_run_rscs.append(resource_id)
                rsc_info[resource_id] = resource_info
        else:
            running_node = []
            resource_info = {}
            resource_info["status"] = "Not Running"
            resource_info["status_messafe"] = ""
            resource_info["status"] = get_rsc_status(resources_primitive)
            if "node" in resources_primitive:
                nodes = resources_primitive['node']
                if isinstance(nodes, list):
                    for node in nodes:
                        nodename = node['@name']
                        running_node.append(nodename)
                else:
                    nodename = nodes['@name']
                    running_node.append(nodename)
            resource_info["running_node"] = running_node
            resource_id = resources_primitive["@id"]
            all_run_rscs.append(resource_id)
            rsc_info[resource_id] = resource_info
    return rsc_info


def get_cluster_status():
    status, output = common_utils.run_cmd("crm_mon -1")
    return status


# status_messesge 资源的状态信息，如果资源出现异常时，会出现该错误信息
# status 资源运行状态
# running_node 运行节点


def get_resource_info():
    # 如果集群未运行，返回数据为空
    cluster_status = get_cluster_status()
    if cluster_status != 0:
        return {"action": True, "data": []}
    constraints = get_all_constraints()  # /get_all_constraints
    if not constraints['action']:
        return json.dumps(constraints)
    re_all_rsc_status = {}
    re_all_rsc_status['data'] = []
    for constraint in constraints['data']:
        rsc_id = str(constraint['id'])
        # 添加是否允许回迁操作
        migrate_rscs = get_all_migrate_rscs()
        if rsc_id in migrate_rscs:
            constraint['allow_unmigrate'] = True
        else:
            constraint['allow_unmigrate'] = False
        type = get_rsc_type(rsc_id)
        subrscs = get_sub_rscs(rsc_id)
        if type in ['group', 'clone']:
            constraint['subrscs'] = subrscs['subrscs']
        else:
            constraint['svc'] = get_resource_svc(rsc_id)
        constraint['type'] = type

    return constraints


def get_resource_svc(rsc_id):
    cmd = "crm_resource --resource " + str(rsc_id) + " --query-xml"
    status, output = common_utils.run_cmd(cmd)
    if status == 0:
        xml_str = output.split("xml:")[1]
        json_str = json.dumps(xmltodict.parse(xml_str))
        json_dict = json.loads(json_str)
        res_type = json_dict["primitive"]["@type"]
        return res_type
    return ""


def resource_action(rsc_id, action, data):
    # 单个资源操作
    """
    operations: start stop delete cleanup
    Example input:
    {}
    """
    # 防止clone资源会出现资源名称有冒号
    rsc_id = rsc_id.split(":")[0]
    cmd = "crm_resource --resource "
    if str(action) == "start":
        if rsc_id.endswith('-clone'):
            rsc_id = str(rsc_id)
            cmd_str = "pcs resource enable " + str(rsc_id)
            status, output = common_utils.run_cmd(cmd_str)
            if status != 0:
                return {'action': False, 'error': _(output)}
            return {"action": True, "info": _("Action on resource success")}
        else:
            cmd_str = "pcs resource enable " + str(rsc_id)
            status, output = common_utils.run_cmd(cmd_str)
            if status != 0:
                return {'action': False, 'error': _(output)}
            return {"action": True, "info": _("Action on resource success")}
    if str(action) == "stop":
        if rsc_id.endswith('-clone'):
            rsc_id = str(rsc_id)
            cmd_str = "pcs resource disable " + str(rsc_id)
            status, output = common_utils.run_cmd(cmd_str)
            if status != 0:
                return {'action': False, 'error': _(output)}
            return {"action": True, "info": _("Action on resource success")}
        else:
            cmd_str = "pcs resource disable " + str(rsc_id)
            status, output = common_utils.run_cmd(cmd_str)
            if status != 0:
                return {'action': False, 'error': _(output)}
            return {"action": True, "info": _("Action on resource success")}
    if str(action) == "delete":
        # 通过id查询category
        if get_rsc_category(rsc_id) == "group":
            cmd_str = "pcs resource delete " + str(rsc_id) + " --force"
        elif get_rsc_category(rsc_id) == "clone":
            clone_str = str(rsc_id[:-6])
            cmd_str = "pcs resource delete" + " " + clone_str + " --force"
        else:
            cmd_str = "pcs resource delete " + str(rsc_id) + " --force"
        status, output = common_utils.run_cmd(cmd_str)
        if status != 0:
            return {'action': False, 'error': _(output)}
        return {"action": True, "info": _("Delete resource success")}
    if str(action) == "cleanup":
        cmd_str = cmd + str(rsc_id) + " --cleanup"
        status, output = common_utils.run_cmd(cmd_str)
        if status != 0:
            return {'action': False, 'error': _(output)}
        return {"action": True, "info": _("Clean up resource success")}
    """
    Example input:
    migrate：
    {"is_force": true, "to_node": "ns187", "period": "PYMDTHM3S"}
    """
    if str(action) == "migrate":
        # 迁移节点
        to_node = ""
        to_node = str(data["to_node"])  # 迁移到指定节点
        cmd_head = "crm_resource --resource "
        cmd_str = cmd_head + str(rsc_id) + " --move -N " + to_node
        status, output = common_utils.run_cmd(cmd_str)
        # if str(to_node) in manager.get_rsc_running_on(str(rsc_id)):
        #     return {'action': False, 'error': _("The resource %s is running on node %s already!") % (str(rsc_id), str(to_node))}
        if status != 0:
            if output == "Error performing operation: Situation already as requested":
                return {
                    'action':
                    False,
                    'error':
                    _("The resource %s is running on node %s already!") %
                    (str(rsc_id), str(to_node))
                }
            else:
                return {'action': False, 'error': _(output)}
        return {"action": True, "info": _("Migrate resource success")}
    """
    unmigrate: 
    {"rsc_id": "kk1", "is_all_rscs": false}
    """
    if str(action) == "unmigrate":
        # 迁离当前节点
        cmd_str = "cibadmin --query --scope constraints"
        status, output = common_utils.run_cmd(cmd_str)
        etree = ET.fromstring(output)  # 封装
        rsc_name = etree.findall("./rsc_location")
        rsc_data = []
        for i in rsc_name:
            rsc = {"rsc": i.get("rsc", ""), "rsc_id": i.get("id", "")}
            rsc_data.append(rsc)
            if rsc_id == rsc["rsc"]:
                # pcs constraint location delete <location_id>
                location_id = str(rsc["rsc_id"])
                cmd_str = "pcs constraint location delete " + location_id
                status, output = common_utils.run_cmd(cmd_str)
            # else:
            #     return {'action': False, 'error': "rsc_id unfound"}
        if status != 0:
            return {'action': False, 'error': (output)}
        return {"action": True, "info": _("Unmigrate resource success")}
    """
    location:
    {"node_level": [{"node": "ns187", "level": "Master Node"}, 
    {"node": "ns188", "level": "Slave 1"}]}
    """
    if str(action) == "location":
        # 删除所有旧的location约束
        ids = []
        ids = get_rsc_constraint_id(rsc_id, action)
        for item in ids:
            cmd_str = "pcs constraint location delete " + str(item)
            status, output = common_utils.run_cmd(cmd_str)
            if status != 0:
                return {'action': False, 'error': (output)}
        node = ""
        score = 0
        # 根据主从给节点赋值
        for item in data["node_level"]:
            if item.get("level", "") == "Master Node":
                # master score = 20000
                score = 20000
            elif item.get("level", "") == "Slave 1":
                score = 16000
            node = str(item.get("node", ""))
            cmd_str = "pcs constraint location " + str(
                rsc_id) + " prefers " + node + "=" + str(score)
            status, output = common_utils.run_cmd(cmd_str)
            if status != 0:
                return {'action': False, 'error': (output)}
        return {"action": True, "info": _("Action on resource success")}
    """
    colocation:
    {"same_node": ["test1234"],"diff_node": ["group_tomcat"]}
    """
    if str(action) == "colocation":
        # 删除所有旧约束
        target_id = get_rsc_constraint_id(rsc_id, action)
        del_colocation_by_id_action(rsc_id, target_id)
        sn = data["same_node"]
        dn = data["diff_node"]
        # 协同节点
        for item in sn:
            cmd_str = "pcs constraint colocation add " + str(
                rsc_id) + " with " + item + " INFINITY"
            status, output = common_utils.run_cmd(cmd_str)
            if status != 0:
                return {'action': False, 'error': _(output)}
        # 非协同节点
        for item in dn:
            cmd_str = "pcs constraint colocation add " + str(
                rsc_id) + " with " + item + " -INFINITY"
            status, output = common_utils.run_cmd(cmd_str)
            if status != 0:
                return {'action': False, 'error': (output)}
    """
    order:
    {"before_rscs": ["test4","test5"],"after_rscs": ["group1"]}
    """
    if str(action) == "order":
        # 删除所有之前的order约束
        if find_order(rsc_id):
            status, output = common_utils.run_cmd(
                "pcs constraint order delete " + str(rsc_id))
            if status != 0:
                return {'action': False, 'error': _(output)}
        # 提取信息并创建新的order约束
        br = data["before_rscs"]
        ar = data["after_rscs"]
        rsc_id = str(rsc_id)
        for i in br:
            cmd_br = "pcs constraint order start " + str(i) + " then " + str(
                rsc_id)
            status, output = common_utils.run_cmd(cmd_br)
            if status != 0:
                return {'action': False, 'error': (output)}
        for j in ar:
            cmd_ar = "pcs constraint order start " + str(
                rsc_id) + " then " + str(j)
            status, output = common_utils.run_cmd(cmd_ar)
            if status != 0:
                return {'action': False, 'error': (output)}
    return {"action": True, "info": _("Action on resource success")}


def find_order(rsc_id):
    cmd_str = "cibadmin --query --scope constraints"
    status, output = common_utils.run_cmd(cmd_str)
    if status != 0:
        return {'action': False, 'error': (output)}
    etree = ET.fromstring(str(output))
    et = etree.findall("./rsc_order")
    for item in et:
        first = item.get("first", "")
        after = item.get("then", "")
        # 判断跟rsc_id是否有关系
        if str(first) == str(rsc_id) or str(after) == str(rsc_id):
            return True
    return False


def update_resource_attributes(rsc_id, data):
    # 修改资源meta_attributes和instance_attributes属性
    """
    Example data:
    {
        "category": "primitive",
        "actions":[
                {
                    "interval":"100",
                    "name":"start"
                }
            ],
        "meta_attributes":{
            "resource-stickiness":"104",
            "is-managed":"true",
            "target-role":"Started"
        },
        "type":"Filesystem",
        "id":"iscisi",
        "provider":"heartbeat",
        "instance_attributes":{
            "device":"/dev/sda1",
            "directory":"/var/lib/mysql",
            "fstype":"ext4"
        },
        "class":"ocf"
    }
    """
    status = ""
    output = ""
    if data == {} or data is None:
        return {'action': False, 'error': ('No input data')}
    # 删除之前所有的属性
    attrib = []
    attrib = get_meta_and_inst(rsc_id)
    if "meta_attributes" in attrib:
        for i in attrib["meta_attributes"]:
            cmd_str = "crm_resource -r " + str(
                rsc_id) + " -m --delete-parameter " + str(i)
            common_utils.run_cmd(cmd_str)
    if "instance_attributes" in attrib:
        for i in attrib["instance_attributes"]:
            cmd_str = "crm_resource -r " + str(
                rsc_id) + " --delete-parameter " + str(i)
            common_utils.run_cmd(cmd_str)
    if data["category"] == "group":
        if "meta_attributes" in data:
            meta = data["meta_attributes"]
            for key in meta.keys():
                value = meta[key]
                # cmd_str = "crm_resource -m -r " + str(rsc_id) + " -p " + str(
                #     key) + " -v " + str(value)
                cmd_str = "pcs resource meta " + str(
                    rsc_id) + " " + str(key) + "=" + str(value)
                status, output = common_utils.run_cmd(cmd_str)
    else:
        if "meta_attributes" in data:
            meta = data["meta_attributes"]
            for key in meta.keys():
                value = meta[key]
                cmd_str = "pcs resource update " + str(
                    rsc_id) + " meta " + str(key) + "=" + str(value)+ ' --force'
                status, output = common_utils.run_cmd(cmd_str)

    if "instance_attributes" in data:
        inst = data["instance_attributes"]
        inst_str = ""
        for key, value in inst.items():
            inst_str = inst_str + str(key) + "=" + str(value) + " "
    #fix error of resource name
    cmd_str = "sleep 1"
    status, output = common_utils.run_cmd(cmd_str)

    cmd_str = "pcs resource update " + str(rsc_id) + " " + inst_str + " --force"
    status, output = common_utils.run_cmd(cmd_str)

    # #修改operation操作属性
    if "actions" in data:
        # 删除之前的所有操作属性
        op_list = get_all_ops(rsc_id)
        if op_list:
            cmd_del_head = "pcs resource op delete " + str(rsc_id)
            for op in op_list:
                cmd_del = cmd_del_head + " " + str(op)
                status, output = common_utils.run_cmd(cmd_del)
        action = data["actions"]  # ist
        # 重写
        cmd_in = "pcs resource update " + str(rsc_id) + " op"
        for ops in action:
            name = str(ops["name"])
            cmd_in = cmd_in + " " + name
            if "interval" in ops:
                cmd_in = cmd_in + " " + "interval=" + str(ops["interval"])
            if "start-delay" in ops:
                cmd_in = cmd_in + " " + "start-delay=" + str(
                    ops["start-delay"])
            if "timeout" in ops:
                cmd_in = cmd_in + " " + "timeout=" + str(ops["timeout"])
            if "role" in ops:
                cmd_in = cmd_in + " " + "role=" + str(ops["role"])
            if "requires" in ops:
                cmd_in = cmd_in + " " + "requires=" + str(ops["requires"])
            if "on-fail" in ops:
                cmd_in = cmd_in + " " + "on-fail=" + str(ops["on-fail"])
        status, output = common_utils.run_cmd(cmd_in)

    if data["category"] == "group":
        # 组资源加入新资源
        """
        {
            "id":"group1",
            "category":"group",
            "rscs":["iscisi", "test1" ],
            "meta_attributes":{
                "target-role":"Started"
            }
        }
        """
        # 获取新的组资源列表
        rscs = data["rscs"]
        # 获取group资源已有子资源列表
        rscs_exist = get_group_rscs(rsc_id)
        cmd_head = "pcs resource group add " + str(rsc_id)
        # 单独对新列表第一项进行判断操作
        # 如果第一项已经存在
        if rscs[0] in rscs_exist:
            # 在rscs_exist中移出该项
            rscs_exist.remove(rscs[0])
        else:
            # 如果第一项不存在，则添加第一项后进行删除和添加操作
            del_pri_attrib(rscs[0])  # 删除资源约束条件
            status, output = common_utils.run_cmd(cmd_head + " " +
                                                  str(rscs[0]))
            if status != 0:
                return {'action': False, 'error': (output)}
        rscs.remove(rscs[0])
        # 针对rscs_exist的删除操作和rscs的添加操作
        # 将rscs_exist中的资源全部删除
        cmd_rmv = ""
        cmd_rmv_head = "pcs resource group remove " + str(rsc_id)
        for i in rscs_exist:
            cmd_rmv = cmd_rmv_head + " " + str(i)
            status, output = common_utils.run_cmd(cmd_rmv)
            if status != 0:
                return {'action': False, 'error': (output)}
        # 将rscs中的资源全部添加
        cmd_add = ""
        for i in rscs:
            del_pri_attrib(i)  # 删除资源约束条件
            cmd_add = cmd_head + " " + str(i)
            status, output = common_utils.run_cmd(cmd_add)
            if status != 0:
                return {'action': False, 'error': (output)}
    return {'action': True, 'info': _("Update resource attributes Success")}


# 获取rsc已有的op列表(仅属性名，如start)
def get_all_ops(rsc_id):
    op_list = []
    cmd_str = "crm_resource --resource " + str(rsc_id) + " --query-xml"
    status, output = common_utils.run_cmd(cmd_str)  # 输出为xml格式
    # format
    xml_data = output.split(":\n")
    xml = xml_data[1]
    etree = ET.fromstring(xml)
    e = etree.find("operations")
    if e:
        op = e.findall("./op")
        for item in op:
            op_list.append(item.get("name", ""))
    return op_list


# 获取组资源的rscs列表
def get_group_rscs(group_id):
    cmd_str = "crm_resource --resource " + str(group_id) + " --query-xml"
    status, output = common_utils.run_cmd(cmd_str)  # 输出为xml格式
    if status != 0:
        return {'action': False, 'error': _(output)}
    # format
    xml_data = output.split(":\n")
    xml = xml_data[1]
    etree = ET.fromstring(xml)
    et = etree.findall("./primitive")
    rscs = []
    for pri in et:
        rscs.append(pri.get("id", ""))
    return rscs


# 获取资源category(primitive, group, clone)
def get_rsc_category(rsc_id):
    cmd_str = "crm_resource --resource " + str(rsc_id) + " --query-xml"
    status, output = common_utils.run_cmd(cmd_str)  # 输出为xml格式
    if status != 0:
        return {'action': False, 'error': _(output)}
    # format
    xml_data = output.split(":\n")
    xml = xml_data[1]
    etree = ET.fromstring(xml)
    ct = str(etree.tag)
    return ct


def del_clone_attrib(rsc_id):
    attrib = []
    attrib = get_meta_and_inst(rsc_id)
    if "meta_attributes" in attrib:
        for i in attrib["meta_attributes"]:
            cmd_str = "crm_resource -r " + str(
                rsc_id) + " -m --delete-parameter " + str(i)
            common_utils.run_cmd(cmd_str)


def del_pri_attrib(rsc_id):
    # 删除之前所有的属性
    attrib = []
    attrib = get_meta_and_inst(rsc_id)
    if attrib == {}:
        pass
    if "meta_attributes" in attrib:
        for i in attrib["meta_attributes"]:
            if i == "is-managed":
                cmd_str = "crm_resource -r " + str(
                    rsc_id) + " -m --delete-parameter " + "is-managed"
                common_utils.run_cmd(cmd_str)
            if i == "priority":
                cmd_str = "crm_resource -r " + str(
                    rsc_id) + " -m --delete-parameter " + "priority"
                common_utils.run_cmd(cmd_str)
            if i == "target-role":
                cmd_str = "crm_resource -r " + str(
                    rsc_id) + " -m --delete-parameter " + "target-role"
                common_utils.run_cmd(cmd_str)
    # attrib = []
    # attrib = get_meta_and_inst(rsc_id)
    # if "meta_attributes" in attrib:
    #     for i in attrib["meta_attributes"]:
    #         cmd_str = "crm_resource -r " + str(
    #             rsc_id) + " -m --delete-parameter " + str(i)
    #         common_utils.run_cmd(cmd_str)

    # if "instance_attributes" in attrib:
    #     for i in attrib["instance_attributes"]:
    #         cmd_str = "crm_resource -r " + str(
    #             rsc_id) + " --delete-parameter " + str(i)
    #         common_utils.run_cmd(cmd_str)

    # 删除所有约束条件
    # colocation
    target_id = get_rsc_constraint_id(rsc_id, "colocation")
    del_colocation_by_id_action(rsc_id, target_id)
    # location
    ids = []
    ids = get_rsc_constraint_id(rsc_id, "location")
    for item in ids:
        cmd_str = "pcs constraint location delete " + str(item)
        status, output = common_utils.run_cmd(cmd_str)
        if status != 0:
            return {'action': False, 'error': _(output)}
    # order
    if find_order(rsc_id):
        status, output = common_utils.run_cmd("pcs constraint order delete " +
                                              str(rsc_id))
        if status != 0:
            return {'action': False, 'error': _(output)}


def create_resource(data):
    # 创建新资源，分三种：primitive, group, clone
    """
    {
        "category": "primitive",     //资源范畴
        "meta_attributes":{
            "target-role":"Stopped"
        },
        "type":"CTDB",                     //资源类型
        "class":"ocf",
        "provider":"heartbeat",
        "instance_attributes":{
            "ctdb_recovery_lock":"lock"
        },
        "id":"test1"
    }
    """
    if data == {} or data is None:
        return {'action': False, 'error': _('No input data')}
    rsc_id = str(data["id"])
    cate = str(data["category"])
    # 判断category
    if cate == "primitive":
        rsc_id_str = " id=\"" + rsc_id + "\""
        status = ""
        output = ""
        rsc_class = " class=\"" + str(data["class"]) + "\""
        rsc_type = " type=\"" + str(data["type"]) + "\""
        cmd = "cibadmin --create -o resources --xml-text '<"
        role = "pcs resource disable "
        status, output = common_utils.run_cmd("cibadmin -Q")
        xml_resource = parseString(output).documentElement
        for primitive in xml_resource.getElementsByTagName("primitive"):
            id = str(primitive.getAttribute("id"))
            resource_id = {'id': id}
            if rsc_id is resource_id['id'] or rsc_id == resource_id['id']:
                return {
                    'action': False,
                    'error': " %s 此资源名已经存在" % (str(rsc_id))
                }
        for group in xml_resource.getElementsByTagName("group"):
            id = str(group.getAttribute("id"))
            resource_id = {'id': id}
            if rsc_id is resource_id['id'] or rsc_id == resource_id['id']:
                return {
                    'action': False,
                    'error': " %s 此资源名已经存在" % (str(rsc_id))
                }
        for clone in xml_resource.getElementsByTagName("clone"):
            id = str(clone.getAttribute("id"))
            resource_id = {'id': id}
            if rsc_id is resource_id['id'] or rsc_id == resource_id['id']:
                return {
                    'action': False,
                    'error': " %s 此资源名已经存在" % (str(rsc_id))
                }
        if "provider" in data:
            provider = " provider=\"" + str(data["provider"]) + "\""
            cmd_str = cmd + cate + rsc_id_str + rsc_class + rsc_type + provider + ">'"
            status, output = common_utils.run_cmd(cmd_str)
            if status != 0:
                return {'action': False, 'error': _(output)}
            else:
                flag = 0
                update_resource_attributes(rsc_id, data)
                attrib = get_meta_and_inst(rsc_id)
                for value in attrib.values():
                    if 'target-role' in value:
                        flag = 1
                        pass

                if flag == 0:
                    cmd_value = role + rsc_id
                    status, output = common_utils.run_cmd(cmd_value)
                    if status != 0:
                        return {'action': False, 'error': _(output)}
        else:
            cmd_str = cmd + cate + rsc_id_str + rsc_class + rsc_type + ">'"
            status, output = common_utils.run_cmd(cmd_str)
            if status != 0:
                return {'action': False, 'error': _(output)}
            else:
                flag = 0
                update_resource_attributes(rsc_id, data)
                attrib = get_meta_and_inst(rsc_id)
                for value in attrib.values():
                    if 'target-role' in value:
                        flag = 1
                        pass

                if flag == 0:
                    cmd_value = role + rsc_id
                    status, output = common_utils.run_cmd(cmd_value)
                    if status != 0:
                        return {'action': False, 'error': _(output)}
            # crm_resource -m -r nfs12 --set-parameter target-role --parameter-value stopped
            if status != 0:
                return {'action': False, 'error': _(output)}
            instance = "crm_resource --resource "
            class_ = data['class']
            if class_ == "stonith":
                cmd = instance + str(
                    rsc_id
                ) + " --set-parameter pcmk_host_check --parameter-value static-list"
                status, output = common_utils.run_cmd(cmd)
                if status != 0:
                    return {'action': False, 'error': _(output)}

    elif cate == "group":
        """
        {
            "category": "group",
            "id":"tomcat_group",
            "rscs":[
                    "tomcat6",
                    "tomcat7"
            ],
                        "meta_attributes":{ 
                "target-role":"Stopped"
            }
        }
        """
        rscs = data["rscs"]
        role = "pcs resource disable "
        # 删除primitive资源的所有属性
        for rsc in rscs:
            del_pri_attrib(rsc)
        rsc_id = str(data["id"])
        cmd_str = "pcs resource group add " + rsc_id
        for r in rscs:
            cmd_str = cmd_str + " " + str(r)
            status, output = common_utils.run_cmd("cibadmin -Q")
            xml_resource = parseString(output).documentElement
            for primitive in xml_resource.getElementsByTagName("primitive"):
                id = str(primitive.getAttribute("id"))
                resource_id = {'id': id}
                if rsc_id is resource_id['id'] or rsc_id == resource_id['id']:
                    return {
                        'action': False,
                        'error': " %s 此资源名已经存在" % (str(rsc_id))
                    }
            for group in xml_resource.getElementsByTagName("group"):
                id = str(group.getAttribute("id"))
                resource_id = {'id': id}
                if rsc_id is resource_id['id'] or rsc_id == resource_id['id']:
                    return {
                        'action': False,
                        'error': " %s 此资源名已经存在" % (str(rsc_id))
                    }
            for clone in xml_resource.getElementsByTagName("clone"):
                id = str(clone.getAttribute("id"))
                resource_id = {'id': id}
                if rsc_id is resource_id['id'] or rsc_id == resource_id['id']:
                    return {
                        'action': False,
                        'error': " %s 此资源名已经存在" % (str(rsc_id))
                    }
        status, output = common_utils.run_cmd(cmd_str)
        if status != 0:
            return {'action': False, 'error': _(output)}
        else:
            flag = 0
            update_resource_attributes(rsc_id, data)
            attrib = get_meta_and_inst(rsc_id)
            for value in attrib.values():
                if 'target-role' in value:
                    flag = 1
                    pass

            if flag == 0:
                cmd_value = role + rsc_id
                status, output = common_utils.run_cmd(cmd_value)
                if status != 0:
                    return {'action': False, 'error': _(output)}
    elif cate == "clone":
        """
        {
            "category": "clone",
            "id":"test5",
            "rsc_id":"test4",
            "meta_attributes":{
                "target-role":"Stopped"
            }
        }
        *id is unused
        """
        ori_id = str(data["rsc_id"])
        ids = get_rsc_constraint_id(ori_id, "location")
        for item in ids:
            cmd_str = "pcs constraint location delete " + str(item)
            status, output = common_utils.run_cmd(cmd_str)
            if status != 0:
                return {'action': False, 'error': _(output)}
        role = "pcs resource disable "
        cmd_str = "pcs resource clone " + ori_id
        del_clone_attrib(ori_id)
        status, output = common_utils.run_cmd(cmd_str)
        if status != 0:
            return {'action': False, 'error': _(output)}
        else:
            flag = 0
            update_resource_attributes(rsc_id, data)
            attrib = get_meta_and_inst(rsc_id)
            for value in attrib.values():
                if 'target-role' in value:
                    flag = 1
                    pass

            if flag == 0:
                cmd_value = role + rsc_id
                status, output = common_utils.run_cmd(cmd_value)
                if status != 0:
                    return {'action': False, 'error': _(output)}
    return {
        "action": True,
        "info": _("Add " + str(cate) + " resource success")
    }


def unmigrate_rsc(is_all_rscs, rsc_id):
    # 保留标签 is_all_rscs = False
    rsc_locations = {}
    for rsc_location in get_all_migrate_rscs():
        id = str(rsc_location.getAttribute("id"))
        rsc_locations[id] = rsc_location
    migrate_ids = {}
    status = ""
    output = ""
    for id in rsc_locations:
        for prefix in ["cli-prefer-", "cli-standby-"]:
            if id.startswith(prefix):
                split_id = id.split(prefix)
                if len(split_id) > 1:
                    tmp_rsc_id = split_id[1]
                    if not tmp_rsc_id in migrate_ids:
                        migrate_ids[tmp_rsc_id] = []
                    migrate_ids[tmp_rsc_id].append(id)
    if is_all_rscs:
        for rsc_id in migrate_ids.keys():
            for id in migrate_ids[rsc_id]:
                xml_str = rsc_locations[id].toxml().replace("\n", "")
                status, output = common_utils.run_cmd(
                    "cib_delete\nconstraints\n%s" % (str(xml_str)))
    else:
        if rsc_id not in migrate_ids.keys():
            return {'action': True, 'info': _("Unmigrate resource success")}
        for id in migrate_ids[rsc_id]:
            xml_str = rsc_locations[id].toxml().replace("\n", "")
            status, output = common_utils.run_cmd(
                "cib_delete\nconstraints\n%s" % (str(xml_str)))

    if status != 0:
        return {'action': False, 'error': _(output)}
    else:
        return {'action': True, 'info': _('Unmigrate resource success')}


def get_all_rsc_metas():
    # 获取支持的资源代理标准
    standards = []
    status, output = common_utils.run_cmd("crm_resource --list-standards")
    if status != 0:
        return {'action': False, 'error': _(output)}
    standards = str(output).split("\n")
    data = {}
    res = {}
    for st in standards:
        if st == "ocf":
            # 获取providers列表
            status, output = common_utils.run_cmd(
                "crm_resource --list-ocf-providers")
            if status != 0:
                return {'action': False, 'error': _(output)}
            pvds = []
            pvds = str(output).split("\n")
            # 根据providers列出所有资源
            for p in pvds:
                status, output = common_utils.run_cmd(
                    "crm_resource --list-agents ocf:" + str(p))
                if status != 0:
                    return {'action': False, 'error': _(output)}
                ag = []
                ag = str(output).split("\n")
                # 去除重复项
                ag = list(set(ag))
                res[str(p)] = ag
                # 去除重复项
            data["ocf"] = res
        # lsb目前没有
        elif st == "lsb":
            continue
        else:
            status, output = common_utils.run_cmd(
                "crm_resource --list-agents " + st)
            if status != 0:
                return {'action': False, 'error': _(output)}
            la = []
            la = str(output).split("\n")
            # 去除重复项
            la = list(set(la))
            for subla in la[::-1]:
                if subla.endswith('@'):
                    la.remove(subla)
            for subla in la:
                if re.match('.*\@', subla):
                    la.remove(subla)
            data[st] = la
    return {'action': True, 'data': data}


# 获取资源元属性
def get_rsc_meta_attributes(category):
    '''
{
    "action": true,
    "data": {
        "target-role": {
            "content": {
                "values": [
                    "Stopped",
                    "Started"
                ],
                "default": "Stopped",
                "type": "enum",
                "dec": "What state should the cluster attempt to keep this resource in?"
            },
            "name": "target-role"
        },
        "priority": {
            "content": {
                "type": "integer",
                "dec": "If not all resources can be active, the cluster will stop lower priority resources in order to keep higher priority ones active."
            },
            "name": "priority"
        },
        "is-managed": {
            "content": {
                "type": "boolean",
                "dec": "Is the cluster allowed to start and stop the resource?"
            },
            "name": "is-managed"
        }
    }
}
    '''
    retjson = {}

    retjson['target-role'] = {}
    retjson['target-role']['content'] = {}
    retjson['target-role']['name'] = 'target-role'
    retjson['target-role']['content']['values'] = ['Stopped', 'Started']
    retjson['target-role']['content']['default'] = 'Stopped'
    retjson['target-role']['content']['type'] = "enum"
    retjson['target-role']['content'][
        'dec'] = "What state should the cluster attempt to keep this resource in?"

    retjson['priority'] = {}
    retjson['priority']['content'] = {}
    retjson['priority']['name'] = 'priority'
    retjson['priority']['content']['type'] = "integer"
    retjson['priority']['content'][
        'dec'] = "If not all resources can be active, the cluster will stop lower priority resources in order to keep higher priority ones active."

    retjson['is-managed'] = {}
    retjson['is-managed']['content'] = {}
    retjson['is-managed']['name'] = 'is-managed'
    # retjson['is-managed']['content']['values'] = ['true', 'false']
    retjson['is-managed']['content']['type'] = "boolean"
    retjson['is-managed']['content'][
        'dec'] = "Is the cluster allowed to start and stop the resource?"

    if category == "group":
        return {'action': True, 'data': retjson}

    retjson['resource-stickiness'] = {}
    retjson['resource-stickiness']['content'] = {}
    retjson['resource-stickiness']['name'] = 'resource-stickiness'
    retjson['resource-stickiness']['content']['type'] = "integer"
    retjson['resource-stickiness']['content'][
        'dec'] = "How much does the resource prefer to stay where it is? Defaults to the value of \'default-resource-stickiness\'"

    retjson['migration-threshold'] = {}
    retjson['migration-threshold']['content'] = {}
    retjson['migration-threshold']['name'] = 'migration-threshold'
    retjson['migration-threshold']['content']['type'] = "integer"
    retjson['migration-threshold']['content'][
        'dec'] = "How many failures should occur for this resource on a node before making the node ineligible to host this resource. Default: \'none\'"

    retjson['multiple-active'] = {}
    retjson['multiple-active']['content'] = {}
    retjson['multiple-active']['name'] = 'multiple-active'
    retjson['multiple-active']['content']['values'] = [
        'stop_start', 'stop_only', 'block'
    ]
    retjson['multiple-active']['content']['type'] = "enum"
    retjson['multiple-active']['content'][
        'dec'] = "What should the cluster do if it ever finds the resource active on more than one node."

    retjson['failure-timeout'] = {}
    retjson['failure-timeout']['content'] = {}
    retjson['failure-timeout']['name'] = 'failure-timeout'
    retjson['failure-timeout']['content']['type'] = "integer"
    retjson['failure-timeout']['content'][
        'dec'] = "How many seconds to wait before acting as if the failure had not occurred (and potentially allowing the resource back to the node on which it failed. Default: \'never\'"

    retjson['allow-migrate'] = {}
    retjson['allow-migrate']['content'] = {}
    retjson['allow-migrate']['name'] = 'allow-migrate'
    # retjson['allow-migrate']['content']['values'] = ['true', 'false']
    retjson['allow-migrate']['content']['type'] = "boolean"
    retjson['allow-migrate']['content'][
        'dec'] = "Allow resource migration for resources which support migrate_to/migrate_from actions."

    if category == "primitive":
        return {'action': True, 'data': retjson}

    if category == "clone":
        retjson['interleave'] = {}
        retjson['interleave']['content'] = {}
        retjson['interleave']['name'] = 'interleave'
        retjson['interleave']['content']['type'] = "boolean"
        retjson['interleave']['content'][
            'dec'] = "Changes the behavior of ordering constraints (between clones/masters) so that instances can start/stop as soon as their peer instance has (rather than waiting for every instance of the other clone has)."

        retjson['clone-max'] = {}
        retjson['clone-max']['content'] = {}
        retjson['clone-max']['name'] = 'clone-max'
        retjson['clone-max']['content']['type'] = "integer"
        retjson['clone-max']['content'][
            'dec'] = "How many copies of the resource to start. Defaults to the number of nodes in the cluster."

        retjson['promoted-max'] = {}
        retjson['promoted-max']['content'] = {}
        retjson['promoted-max']['name'] = 'promoted-max'
        retjson['promoted-max']['content']['type'] = "integer"
        retjson['promoted-max']['content'][
            'dec'] = "If promotable is true, the number of instances that can be promoted at one time across the entire cluster"

        retjson['promotable'] = {}
        retjson['promotable']['content'] = {}
        retjson['promotable']['name'] = 'promotable'
        retjson['promotable']['content']['type'] = "boolean"
        retjson['promotable']['content'][
		'desc'] = "If true, clone instances can perform a special role that Pacemaker will manage via the resource agent's promote and demote actions. The resource agent must support these actions"

        retjson['notify'] = {}
        retjson['notify']['content'] = {}
        retjson['notify']['name'] = 'notify'
        retjson['notify']['content']['type'] = "boolean"
        retjson['notify']['content'][
		'desc'] = "Call the resource agent's notify action for all active instances, before and after starting or stopping any clone instance"

        return {'action': True, 'data': retjson}

    return {'action': True, 'data': retjson}


def get_resource_info_by_rsc_id(rsc_id):
    cmd_str = "crm_resource --resource " + str(rsc_id) + " --query-xml"
    status, output = common_utils.run_cmd(cmd_str)  # 输出为xml格式
    if status != 0:
        return {'action': False, 'error': _(output)}
    # format
    xml_data = output.split(":\n")
    xml = xml_data[1]
    etree = ET.fromstring(xml)
    ct = etree.tag
    data = common_utils.get_resource_info(ct, xml)  # 获取资源信息并按格式输出
    data["id"] = str(rsc_id)
    data["category"] = str(ct)
    ret = {}
    if data is not None:
        ret['data'] = data
        ret['action'] = True
    else:
        ret['action'] = False
        ret['error'] = data

    if 'data' in ret:
        data_res = ret['data']
        if 'provider' in data_res:
            provider = data_res['provider']
            if provider == "":
                del ret['data']['provider']

    return ret


# 获取资源已有元属性和实例属性
def get_meta_and_inst(rsc_id):
    cmd_str = "crm_resource --resource " + str(rsc_id) + " --query-xml"
    status, output = common_utils.run_cmd(cmd_str)  # 输出为xml格式
    if status != 0:
        return {'action': False, 'error': _(output)}
    # format
    xml_data = output.split(":\n")
    xml = xml_data[1]
    etree = ET.fromstring(xml)
    data = {}
    # 获取meta_attributes
    e = etree.find("meta_attributes")
    if e:
        prop = []
        for item in e.findall("./nvpair"):
            prop.append(item.get("name"))
        data["meta_attributes"] = prop
    # 获取instance_attributes
    e = etree.find("instance_attributes")
    if e:
        prop = []
        for item in e.findall("./nvpair"):
            prop.append(item.get("name"))
        data["instance_attributes"] = prop
    return data


# 获取单个资源的约束条件(location, colocation, order)
def get_rsc_constraints(rsc_id, action):
    data = {}
    cmd_str = "cibadmin --query --scope constraints"
    status, output = common_utils.run_cmd(cmd_str)
    if status != 0:
        return {'action': False, 'error': _(output)}
    etree = ET.fromstring(str(output))
    if str(action) == "order":
        et = etree.findall("./rsc_order")
        before_rscs = []
        after_rscs = []
        for item in et:
            first = item.get("first", "")
            after = item.get("then", "")
            # 判断跟rsc_id是否有关系
            if str(first) == str(rsc_id):
                # 获取after_rscs
                after_rscs.append(item.get("then", ""))
            if str(after) == str(rsc_id):
                # 获取before_rscs
                before_rscs.append(item.get("first", ""))
        data["before_rscs"] = before_rscs
        data["after_rscs"] = after_rscs
        data["rsc_id"] = str(rsc_id)
    if str(action) == "location":
        res = []
        et = etree.findall("./rsc_location")
        for item in et:
            rsc = item.get("rsc", "")
            if str(rsc) == str(rsc_id):
                # 避开非协同节点约束
                if "score" in item.keys():
                    if item.get("score", "") == "-INFINITY":
                        continue
                    if item.get("score", "") == "INFINITY":
                        continue
                    # 获取nodeid
                    prop = {}
                    prop["node"] = item.get("node", "")
                    score = item.get("score", "")
                    if score == "20000":
                        prop["level"] = "Master Node"
                    elif score == "16000":
                        prop["level"] = "Slave 1"
                    res.append(prop)
        data["node_level"] = res
        data["rsc_id"] = str(rsc_id)
    if str(action) == "colocation":
        # 协同节点
        et = etree.findall("./rsc_colocation")
        same_node = []
        diff_node = []
        for item in et:
            rsc = item.get("rsc", "")
            rsc_with = item.get("with-rsc", "")
            if str(rsc) == str(rsc_id):
                if "score" in item.keys() and item.get("score",
                                                       "") == "INFINITY":
                    # 协同节点
                    # 获取with-rsc
                    same_node.append(rsc_with)
                elif "score" in item.keys() and item.get("score",
                                                         "") == "-INFINITY":
                    # 非协同节点
                    # 获取with-rsc
                    diff_node.append(rsc_with)
            # 如果是with_node
            if str(rsc_with) == str(rsc):
                if "score" in item.keys() and item.get("score",
                                                       "") == "INFINITY":
                    # 协同节点
                    # 获取with-rsc
                    same_node.append(rsc)
                elif "score" in item.keys() and item.get("score",
                                                         "") == "-INFINITY":
                    # 非协同节点
                    # 获取with-rsc
                    diff_node.append(rsc)
        data["same_node"] = same_node
        data["diff_node"] = diff_node
        data["rsc_id"] = str(rsc_id)
    # 最终数据格式
    ret = {}
    ret["action"] = True
    ret["data"] = data
    return ret


# 按照资源id和action获取相对应的约束条件id（仅location和colocation）
def get_rsc_constraint_id(rsc_id, action):
    cmd_str = "cibadmin --query --scope constraints"
    status, output = common_utils.run_cmd(cmd_str)
    if status != 0:
        return {'action': False, 'error': (output)}
    etree = ET.fromstring(str(output))
    # 按照rsc_id和action获取约束条件id
    ids = []  # 存储约束条件id
    target_id = []  # 存储协同节点id
    if str(action) == "colocation":
        et = etree.findall("./rsc_colocation")
        # 遍历所有order约束查找含有rsc_id的项并添加进target_id列表里
        # 协同节点
        for item in et:
            rsc = item.get("rsc", "")
            rsc_with = item.get("with-rsc", "")
            if str(rsc) == str(rsc_id):
                # 获取with-rsc
                target_id.append(rsc_with)
            if str(rsc_with) == str(rsc):
                target_id.append(rsc)
        return target_id
    if str(action) == "location":
        et = etree.findall("./rsc_location")
        for item in et:
            rsc = item.get("rsc", "")
            if str(rsc) == str(rsc_id):
                # 避开非协同节点约束
                if "score" in item.keys() and item.get("score",
                                                       "") == "-INFINITY":
                    continue
                ids.append(item.get("id", ""))
        return ids
    return []


# 删除source_id的所有colocation约束关系
def del_colocation_by_id_action(source_id, target_id):
    for item in target_id:
        # 遍历于rsc_id有关的目标节点
        cmd_str = "pcs constraint colocation delete " + str(
            source_id) + " " + str(item)
        status, output = common_utils.run_cmd(cmd_str)
        if status != 0:
            return {'action': False, 'error': (output)}


if __name__ == '__main__':

    print(get_all_constraints())
