import os
import resource
import alarm
from flask import Flask
from flask import request, Response
from flask import Flask, session, redirect, url_for, request, render_template, make_response
import cluster
import json
import log
import resource
import gettext
import common_utils
import license
import re
from flask import jsonify
import node
import resource
import scripts
import config
import sys
from datetime import timedelta
import logging

API_VERSION = "v1"
app_name = "ha-api"
gettext.bindtextdomain(app_name)
gettext.textdomain(app_name)
gettext.install(app_name, "/usr/share/locale")
gettext.translation(app_name, "/usr/share/locale",
                    languages=["zh_CN"]).install(True)
sys.path.append("/usr/share/heartbeat-gui/ha-api")


def create_app(test_config=None):
    # create and configure the app
    app = Flask(__name__,
                static_url_path='',
                template_folder='static',
                instance_relative_config=True)

    # Set the secret key to some random bytes. Keep this really secret.
    app.secret_key = b'_5#y2L"F4Q8z\n\xec]/'
    app.config['SECRET_KEY'] = os.urandom(24)
    # Debug INFO
    app.debug = True

    @app.before_request
    def login_check():
        session.permanent = True
        app.config['PERMANENT_SESSION_LIFETIME'] = timedelta(minutes=5)
        user_name = session.get('user_name')
        if user_name is None:
            if not request.path.startswith('/api/v1/login'):
                if request.path.startswith('/api/'):
                    rv = make_response('Session Timeout', 403)
                    return rv

    @app.route('/api/' + API_VERSION + '/login', methods=['POST'])
    def check_auth():
        data = request.json
        if data is not None and 'username' in data and 'password' in data:
            username = data['username']
            if username != 'hacluster':
                return json.dumps({
                    'action':
                    False,
                    'error':
                    _('Username is not allowed to login')
                })
            password = data['password']
            ret = common_utils.check_auth(username, password)
            if ret:
                # 更新 session 信息
                session['user_name'] = username
                session['password'] = password
                return json.dumps({'action': ret, 'info': _('Login success')})
            else:
                return json.dumps({
                    'action': ret,
                    'error': _('Username or password error')
                })
        return json.dumps({'action': False, 'error': _('Login failed')})

    @app.route('/api/' + API_VERSION + '/logout', methods=['POST'])
    def logout():
        session.pop('user_name', None)
        session.pop('password', None)
        return json.dumps({'action': True, 'info': _('Login failed')})

    # a simple page that says hello
    @app.route('/api/' + API_VERSION + '/haclusters/1',
               methods=['GET', 'POST', 'PUT'])
    def cluster_ops():
        if request.method == 'GET':
            #return Response(json.dumps(cluster.get_cluster_properties_info()), mimetype='application/json')
            return json.dumps(cluster.get_cluster_properties_info())
        if request.method == 'PUT':
            data = request.json
            ret = cluster.update_cluster_properties(data)
            return json.dumps(ret)
        else:
            return

    #local HA operation
    @app.route('/api/' + API_VERSION + '/haclusters/1/localnodes/<action>',
               methods=['GET', 'POST', 'PUT'])
    def loocalnodes_ops(action):
        if request.method == 'PUT':
            return json.dumps(cluster.operation_cluster_action(action))

    # Resource ops
    @app.route('/api/' + API_VERSION + '/haclusters/1/resources',
               methods=['GET', 'POST', 'PUT'])
    def resource_ops():
        if request.method == 'GET':
            return json.dumps(resource.get_resource_info())
        if request.method == 'POST':
            #添加资源 primitive group clone
            data = request.json
            ret = json.dumps(resource.create_resource(data))
            return ret
        else:
            return

    #获取所有资源类型
    @app.route('/api/' + API_VERSION + '/haclusters/1/metas',
               methods=['GET', 'POST', 'PUT'])
    def resource_meta_ops():
        if request.method == 'GET':
            return json.dumps(resource.get_all_rsc_metas())
        else:
            return

    #获取资源元属性
    @app.route('/api/' + API_VERSION +
               '/haclusters/1/resources/meta_attributes/<category>',
               methods=['GET', 'POST', 'PUT'])
    def resource_meta_by_cate(category):
        if request.method == 'GET':
            return json.dumps(resource.get_rsc_meta_attributes(category))
        else:
            return

    #修改资源
    @app.route('/api/' + API_VERSION + '/haclusters/1/resources/<rsc_id>',
               methods=['GET', 'POST', 'PUT'])
    def resource_ops_by_id(rsc_id):
        if request.method == 'GET':
            return json.dumps(resource.get_resource_info_by_rsc_id(rsc_id))
        if request.method == 'PUT':
            data = request.json
            ret = resource.update_resource_attributes(rsc_id, data)
        return json.dumps(ret)

    #单个资源操作start、stop、delete、cleanup、migrate、unmigrate、location、order、colocation
    @app.route('/api/' + API_VERSION +
               '/haclusters/1/resources/<rsc_id>/relations/<action>',
               methods=['GET', 'POST', 'PUT'])
    @app.route('/api/' + API_VERSION +
               '/haclusters/1/resources/<rsc_id>/<action>',
               methods=['GET', 'POST', 'PUT'])
    def resource_act(rsc_id, action):
        if request.method == 'GET':
            return json.dumps(resource.get_rsc_constraints(rsc_id, action))
        if request.method == 'PUT':  #按id修改资源
            data = request.json
            ret = resource.resource_action(rsc_id, action, data)
        return json.dumps(ret)

    # hb ops
    @app.route('/api/' + API_VERSION + '/haclusters/1/hbstatus',
               methods=['GET', 'POST', 'PUT'])
    def hb_ops():
        if request.method == 'GET':
            return json.dumps({"action": True, "data": 0})
        if request.method == 'PUT':
            return
        else:
            return

    # config ops
    @app.route('/api/' + API_VERSION + '/haclusters/1/configs',
               methods=['GET', 'POST', 'PUT'])
    def config_ops():
        if request.method == 'GET':
            return json.dumps(config.get_hb_conf())
        if request.method == 'POST':
            data = request.json
            return json.dumps(config.edit_hb_info(data))
            data = {"action": True, "info": "update success"}
            return json.dumps(data)

    # logs ops
    @app.route('/api/' + API_VERSION + '/haclusters/1/logs',
               methods=['GET', 'POST', 'PUT'])
    def log_ops():
        if request.method == 'GET':
            return json.dumps(log.generate_log())
        if request.method == 'PUT':
            return
        else:
            return

    @app.route('/api/' + API_VERSION + '/haclusters/1/commands',
               methods=['GET'])
    def get_commands_list():
        command_table = {
            1: "crm_mon -1 -o",
            2: "crm_simulate -Ls",
            3: "pcs config show",
            4: "corosync-cfgtool -s"
        }
        return json.dumps({'action': True, 'data': command_table})

    @app.route('/api/' + API_VERSION + '/haclusters/1/commands/<cmd_type>',
               methods=['GET'])
    def get_commands_result(cmd_type):
        command_type = int(cmd_type)
        if not isinstance(command_type, int):
            return json.dumps({
                'action': False,
                'error': _("Please check input commands type")
            })

        command_table = {
            0: "All",
            1: "crm_mon -1 -o",
            2: "crm_simulate -Ls",
            3: "pcs config show",
            4: "corosync-cfgtool -s"
        }
        if command_type != 0 and command_type in command_table:
            command_str = command_table[command_type]
            status, output = common_utils.run_cmd(command_str)
            return json.dumps({'action': True, 'data': output})
        else:
            return json.dumps({
                'action': False,
                'error': _("Please check input commands type")
            })

    @app.route('/')
    def send_index_html():
        """
        url mapping for index html file, which with @login_required decorated
        """
        # 检查是否已经登录，如果已登录，则继续；如果未登录则检测URL 地址
        user_name = session.get('user_name')
        if user_name:
            return render_template('index.html')
        return redirect('/login')

    @app.route('/resource')
    def set_resource_html():
        """
        url mapping for index html file, which with @login_required decorated
        """
        return render_template('index.html')

    @app.route('/login')
    def set_login_html():
        """
        url mapping for index html file, which with @login_required decorated
        """
        return render_template('index.html')

    #获取node
    @app.route('/api/' + API_VERSION + '/haclusters/1/nodes/',
               methods=['GET', 'POST', 'PUT'])
    def get_nodes():
        if request.method == 'GET':
            return json.dumps(node.get_node_info())

    #根据id取单个node
    @app.route('/api/' + API_VERSION + '/haclusters/1/nodes/<nodeid>',
               methods=['GET', 'POST', 'PUT'])
    def get_nodeid(nodeid):
        if request.method == 'GET':
            return json.dumps(node.get_nodeid_info(nodeid))

    #node状态修改
    @app.route('/api/' + API_VERSION +
               '/haclusters/1/nodes/<node_id>/<action>',
               methods=['GET', 'POST', 'PUT'])
    def nodes_action(node_id, action):
        if request.method == 'PUT':
            return json.dumps(node.get_nodes_action(node_id, action))

    #获取资源创建
    @app.route('/api/' + API_VERSION +
               '/haclusters/1/metas/<rsc_class>/<rsc_type>',
               methods=['GET', 'POST', 'PUT'])
    @app.route('/api/' + API_VERSION +
               '/haclusters/1/metas/<rsc_class>/<rsc_type>/<rsc_provider>/',
               methods=['GET', 'POST', 'PUT'])
    def get_rsc_meta(rsc_class, rsc_type, rsc_provider=None):
        if request.method == 'GET':
            return json.dumps(
                resource.get_rsc_metas(rsc_class, rsc_type, rsc_provider))

    @app.route('/api/' + API_VERSION +
               '/haclusters/1/resources/meta_attributes/<category>',
               methods=['GET', 'POST', 'PUT'])
    def resource_category(category):
        if request.method == 'GET':
            return json.dumps(resource.get_rsc_meta_attributes(category))

    @app.route('/api/' + API_VERSION + '/haclusters/1/get_diskhb_device',
               methods=['GET'])
    def get_disk_hb_info():
        if request.method == 'GET':
            return json.dumps({"action": True, "data": []})

    @app.route('/api/' + API_VERSION + '/haclusters/1/scripts',
               methods=['POST'])
    def generate_scripts():
        if request.method == 'POST':
            data = request.json
            return scripts.generate_scripts(data)

    @app.route('/api/' + API_VERSION + '/haclusters/1/alarms',
               methods=['POST', 'GET', 'POST'])
    def alrms_config():
        if request.method == 'GET':
            return json.dumps(alarm.alarms_get())
        if request.method == 'POST':
            data = request.json
            return json.dumps(alarm.alarms_set(data))

    @app.route('/api/' + API_VERSION + '/haclusters/1/licenses',
               methods=['POST', 'GET', 'POST'])
    def get_license_info():
        if request.method == 'GET':
            return json.dumps(license.get_ha_host_license())
        if request.method == 'POST':
            data = request.json
            return json.dumps(license.install_ha_host_license_own(data))

    @app.route('/api/' + API_VERSION + '/haclusters/1/rlicenses',
               methods=['POST', 'GET', 'POST'])
    def get_remote_license_info():
        if request.method == "POST":
            data = request.json
            return json.dumps(license.get_remote_ha_host_license(data))

    return app


if __name__ == '__main__':
    app = create_app()

    # LOG 日志
    logging.basicConfig(level=logging.DEBUG,
                   format='%(asctime)s %(levelname)s %(message)s',
                   datefmt='%a, %d %b %Y %H:%M:%S',
                   filename='/var/log/ha-api.log',
                   filemode='w')

    app.run("0.0.0.0",
            "8088",
            ssl_context=('/usr/share/heartbeat-gui/ha-api/.cert.pem',
                         '/usr/share/heartbeat-gui/ha-api/.key.pem'))
